package com.livelike.flutter.livelike_flutter_sdk

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import androidx.annotation.NonNull
import com.google.gson.Gson
import com.livelike.engagementsdk.EngagementSDK
import com.livelike.engagementsdk.EpochTime
import com.livelike.engagementsdk.LiveLikeContentSession
import com.livelike.engagementsdk.LiveLikeWidget
import com.livelike.engagementsdk.chat.LiveLikeChatSession
import com.livelike.engagementsdk.chat.data.remote.LiveLikePagination
import com.livelike.engagementsdk.core.AccessTokenDelegate
import com.livelike.engagementsdk.core.data.models.LeaderBoard
import com.livelike.engagementsdk.core.data.models.LeaderBoardEntry
import com.livelike.engagementsdk.core.data.models.LeaderBoardEntryPaginationResult
import com.livelike.engagementsdk.publicapis.ErrorDelegate
import com.livelike.engagementsdk.publicapis.LiveLikeCallback
import com.livelike.engagementsdk.sponsorship.SponsorModel
import com.livelike.engagementsdk.widget.data.models.WidgetUserInteractionBase
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject


/** LivelikeFlutterSdkPlugin */
class LivelikeFlutterSdkPlugin : FlutterPlugin, MethodCallHandler {

    private lateinit var errorDelegateChannel: EventChannel
    private lateinit var accessTokenDelegateChannel: EventChannel


    /// The MethodChannel that will the communication between Flutter and native Android
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    lateinit var engagementSDK: EngagementSDK
    private var applicationContext: Context? = null
    val sessionMap = hashMapOf<String, LiveLikeContentSession>()
    val chatSessionMap = hashMapOf<String, LiveLikeChatSession>()
    private val uiThreadHandler: Handler = Handler(Looper.getMainLooper())
    private var currentPDTTimeMap = mutableMapOf<String, Long>()

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "livelike_flutter_sdk")
        channel.setMethodCallHandler(this)
        applicationContext = flutterPluginBinding.applicationContext
        flutterPluginBinding.platformViewRegistry
            .registerViewFactory(
                "chatView",
                ChatViewFactory(this, flutterPluginBinding.binaryMessenger)
            )
        flutterPluginBinding.platformViewRegistry
            .registerViewFactory(
                "widgetView",
                WidgetViewFactory(this, flutterPluginBinding.binaryMessenger)
            )
        accessTokenDelegateChannel =
            EventChannel(flutterPluginBinding.binaryMessenger, "flutter_sdk_access_token_delegate")
        errorDelegateChannel =
            EventChannel(flutterPluginBinding.binaryMessenger, "flutter_sdk_error_delegate")

    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
//        Log.d("LiveLike", "Method Call: ${call.method}")
        when (call.method) {
            "getPlatformVersion" -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            "initSDK" -> {
                val clientId: String? = call.argument<String>("clientId")
                clientId?.let {
                    val accessToken: String? = call.argument<String>("accessToken")
                    val usingAccessToken: Boolean? = call.argument<Boolean>("usingAccessToken")
                    val accessTokenEventHandler = LiveLikeFlutterStreamHandler()
                    accessTokenDelegateChannel.setStreamHandler(accessTokenEventHandler)
                    val errorEventHandler = LiveLikeFlutterStreamHandler()
                    errorDelegateChannel.setStreamHandler(errorEventHandler)
                    applicationContext?.let { context ->
                        engagementSDK = EngagementSDK(clientId!!,
                            context,
                            accessTokenDelegate = when (usingAccessToken) {
                                true -> object : AccessTokenDelegate {
                                    override fun getAccessToken(): String? = accessToken

                                    override fun storeAccessToken(accessToken: String?) {
                                        Log.d(
                                            "LiveLike",
                                            "Access Token:$accessToken, Check:${accessTokenEventHandler.eventSink}"
                                        )
                                        accessTokenEventHandler.eventSink?.success(accessToken)
                                    }
                                }
                                else -> null
                            },
                            errorDelegate = object : ErrorDelegate() {
                                override fun onError(error: String) {
                                    errorEventHandler.eventSink?.success(error)
                                }
                            })
                        result.success(null)
                        return
                    }
                    result.error("No Context", "Application Context is null", null)
                    return
                }
                result.error("", "Client Id cannot be empty", null)
            }
            "setVideoTimePDT" -> {
                val videoPDTTime = call.argument<Int>("videoTimePDT")
                val programId = call.argument<String>("programId")
                val chatRoomId = call.argument<String>("chatRoomId")
                programId?.let {
                    currentPDTTimeMap.put(it, videoPDTTime?.toLong() ?: 0L)
                }
                chatRoomId?.let {
                    currentPDTTimeMap.put(it, videoPDTTime?.toLong() ?: 0L)
                }
//                Log.d("LiveLike", "PDT: $videoPDTTime")
            }
            "createSession" -> {
                val programId = call.argument<String>("programId")
                val errorEventHandler = LiveLikeFlutterStreamHandler()
                errorDelegateChannel.setStreamHandler(errorEventHandler)
                programId?.let {
                    sessionMap[programId] = engagementSDK.createContentSession(
                        programId,
                        timecodeGetter = object : EngagementSDK.TimecodeGetter {
                            override fun getTimecode(): EpochTime {
//                                Log.d("LiveLike", "TimeCode")
                                return EpochTime(currentPDTTimeMap[programId] ?: 0L)
                            }
                        },
                        errorDelegate = object : ErrorDelegate() {
                            override fun onError(error: String) {
                                errorEventHandler.eventSink?.success(error)
                            }
                        }).apply {
                        chatSessionMap[this.chatSession.getCurrentChatRoom()] = this.chatSession
                       result.success(
                            hashMapOf(
                                "programId" to this.contentSessionId(),
                                "chatRoomId" to this.chatSession.getCurrentChatRoom()
                            )
                        )
                    }

                    return
                }
                result.error("", "Error in creating the session", null)
            }
            "resumeSession" -> {
                val programId = call.argument<String>("programId")
                programId?.let {
                    sessionMap[programId]?.resume()
                    result.success(null)
                    return
                }
                result.error("", "Error in resuming the session", null)
            }
            "pauseSession" -> {
                val programId = call.argument<String>("programId")
                programId?.let {
                    sessionMap[programId]?.pause()
                    result.success(null)
                    return
                }
                result.error("", "Error in pausing the session", null)
            }
            "closeSession" -> {
                val programId = call.argument<String>("programId")
                programId?.let {
                    sessionMap[programId]?.close()
                    sessionMap.remove(programId)
                    result.success(null)
                    return
                }
                result.error("", "Error in closing the session", null)
            }
            "createChatSession" -> {
                val chatRoomId = call.argument<String>("chatRoomId")
                val errorEventHandler = LiveLikeFlutterStreamHandler()
                errorDelegateChannel.setStreamHandler(errorEventHandler)
                chatRoomId?.let {
                    chatSessionMap[chatRoomId] =
                        engagementSDK.createChatSession(errorDelegate = object : ErrorDelegate() {
                            override fun onError(error: String) {
                                errorEventHandler.eventSink?.success(error)
                            }
                        }, timecodeGetter = object : EngagementSDK.TimecodeGetter {
                            override fun getTimecode(): EpochTime {
                                return EpochTime(currentPDTTimeMap[chatRoomId] ?: 0L)
                            }
                        }).apply {
                            this.connectToChatRoom(
                                chatRoomId,
                                callback = object : LiveLikeCallback<Unit>() {
                                    override fun onResponse(res: Unit?, error: String?) {
                                        uiThreadHandler.post {
                                            res?.let {
                                                result.success(hashMapOf("chatRoomId" to this@apply.getCurrentChatRoom()))
                                            }
                                            error?.let {
                                                result.error("", error, null)
                                            }
                                        }
                                    }
                                })
                        }
                    return
                }
                result.error("", "Error in creating in chat session", null)
            }
            "resumeChatSession" -> {
                val chatRoomId = call.argument<String>("chatRoomId")
                chatRoomId?.let {
                    chatSessionMap[chatRoomId]?.resume()
                    result.success(null)
                    return
                }
                result.error("", "Error in resuming the chat session", null)
            }
            "pauseChatSession" -> {
                val chatRoomId = call.argument<String>("chatRoomId")
                chatRoomId?.let {
                    chatSessionMap[chatRoomId]?.pause()
                    result.success(null)
                    return
                }
                result.error("", "Error in pausing the chat session", null)
            }
            "closeChatSession" -> {
                val chatRoomId = call.argument<String>("chatRoomId")
                chatRoomId?.let {
                    chatSessionMap[chatRoomId]?.close()
                    chatSessionMap.remove(chatRoomId)
                    result.success(null)
                    return
                }
                result.error("", "Error in closing the chat session", null)
            }
            "getLeaderBoards" -> {
                val programId = call.argument<String>("programId")
                programId?.let {
                    engagementSDK.getLeaderBoardsForProgram(programId,
                        object : LiveLikeCallback<List<LeaderBoard>>() {
                            override fun onResponse(list: List<LeaderBoard>?, error: String?) {
                                list?.let {
                                    result.success(list.map {
                                        hashMapOf(
                                            "id" to it.id,
                                            "name" to it.name,
                                            "rewardItem" to hashMapOf(
                                                "id" to it.rewardItem.id,
                                                "name" to it.rewardItem.name
                                            )
                                        )
                                    })
                                }
                            }
                        })
                    return
                }
                result.error("", "Program Id cannot be null", null)
            }
            "getLeaderBoardDetails" -> {
                val leaderBoardId = call.argument<String>("leaderBoardId")
                leaderBoardId?.let {
                    engagementSDK.getLeaderBoardDetails(leaderBoardId, object :
                        LiveLikeCallback<LeaderBoard>() {
                        override fun onResponse(leaderBoard: LeaderBoard?, error: String?) {
                            leaderBoard?.let {
                                result.success(
                                    hashMapOf(
                                        "id" to it.id,
                                        "name" to it.name,
                                        "rewardItem" to hashMapOf(
                                            "id" to it.rewardItem.id,
                                            "name" to it.rewardItem.name
                                        )
                                    )
                                )
                            }
                            error?.let {
                                result.error("", it, null)
                            }
                        }
                    })
                    return
                }
                result.error("", "LeaderBoard Id cannot be null", null)
            }
            "getLeaderBoardEntryForProfile" -> {
                val leaderBoardId = call.argument<String>("leaderBoardId")
                val profileId = call.argument<String>("profileId")
                leaderBoardId?.let {
                    profileId?.let {
                        engagementSDK.getLeaderBoardEntryForProfile(leaderBoardId, profileId,
                            object : LiveLikeCallback<LeaderBoardEntry>() {
                                override fun onResponse(
                                    leaderBoardEntry: LeaderBoardEntry?,
                                    error: String?
                                ) {
                                    leaderBoardEntry?.let {
                                        result.success(
                                            hashMapOf(
                                                "profile_id" to it.profile_id,
                                                "profile_nickname" to it.profile_nickname,
                                                "percentile_rank" to it.percentile_rank.toString(),
                                                "rank" to it.rank,
                                                "score" to it.score.toDouble(),
                                                "profile" to hashMapOf(
                                                    "id" to it.profile.id,
                                                    "nickname" to it.profile.nickname,
                                                    "custom_data" to it.profile.custom_data
                                                )
                                            )
                                        )
                                    }
                                    error?.let {
                                        result.error("", it, null)
                                    }
                                }
                            })
                        return
                    }
                }
                result.error("", "LeaderBoard Id or Profile Id cannot be null", null)
            }
            "getLeaderBoardEntryForCurrentUserProfile" -> {
                val leaderBoardId = call.argument<String>("leaderBoardId")
                leaderBoardId?.let {
                    engagementSDK.getLeaderBoardEntryForCurrentUserProfile(leaderBoardId,
                        object : LiveLikeCallback<LeaderBoardEntry>() {
                            override fun onResponse(
                                leaderBoardEntry: LeaderBoardEntry?,
                                error: String?
                            ) {
                                leaderBoardEntry?.let {
                                    result.success(
                                        hashMapOf(
                                            "profile_id" to it.profile_id,
                                            "profile_nickname" to it.profile_nickname,
                                            "percentile_rank" to it.percentile_rank.toString(),
                                            "rank" to it.rank,
                                            "score" to it.score.toDouble(),
                                            "profile" to hashMapOf(
                                                "id" to it.profile.id,
                                                "nickname" to it.profile.nickname,
                                                "custom_data" to it.profile.custom_data
                                            )
                                        )
                                    )
                                }
                                error?.let {
                                    result.error("", it, null)
                                }
                            }
                        })
                    return
                }
                result.error("", "LeaderBoard Id or Profile Id cannot be null", null)
            }
            "getEntriesForLeaderBoard" -> {
                val leaderBoardId = call.argument<String>("leaderBoardId")
                val pagination = call.argument<String>("pagination")
                leaderBoardId?.let {
                    engagementSDK.getEntriesForLeaderBoard(leaderBoardId, when (pagination) {
                        "first" -> LiveLikePagination.FIRST
                        "previous" -> LiveLikePagination.PREVIOUS
                        "next" -> LiveLikePagination.NEXT
                        else -> LiveLikePagination.FIRST
                    }, object : LiveLikeCallback<LeaderBoardEntryPaginationResult>() {
                        override fun onResponse(
                            paginationResult: LeaderBoardEntryPaginationResult?,
                            error: String?
                        ) {
                            paginationResult?.let { entryPaginationResult ->
                                result.success(hashMapOf(
                                    "count" to entryPaginationResult.count,
                                    "hasNext" to entryPaginationResult.hasNext,
                                    "hasPrevious" to entryPaginationResult.hasPrevious,
                                    "list" to entryPaginationResult.list?.map {
                                        hashMapOf(
                                            "profile_id" to it.profile_id,
                                            "profile_nickname" to it.profile_nickname,
                                            "percentile_rank" to it.percentile_rank.toString(),
                                            "score" to it.score.toDouble(),
                                            "rank" to it.rank,
                                            "profile" to hashMapOf(
                                                "id" to it.profile.id,
                                                "nickname" to it.profile.nickname,
                                                "custom_data" to it.profile.custom_data
                                            )
                                        )
                                    }
                                ))
                            }
                            error?.let {
                                result.error("", it, null)
                            }

                        }
                    })
                    return
                }
                result.error("", "LeaderBoard Id cannot be null", null)
            }
            "fetchSponsorByProgramId" -> {
                val programId = call.argument<String>("programId")
                programId?.let {
                    val sponsor = engagementSDK.sponsor()
                    sponsor.fetchByProgramId(
                        programId,
                        callback = object : LiveLikeCallback<List<SponsorModel>>() {
                            override fun onResponse(list: List<SponsorModel>?, error: String?) {
                                uiThreadHandler.post {
                                    list?.let {
                                        result.success(list.map {
                                            hashMapOf(
                                                "id" to it.id,
                                                "logo_url" to it.logoUrl,
                                                "client_id" to it.clientId,
                                                "name" to it.name
                                            )
                                        })
                                    }
                                    error?.let {
                                        result.error("", it, null)
                                    }
                                }
                            }
                        })
                    return
                }
                result.error("", "Program Id cannot be null or empty", null)
            }
            "fetchWidgetDetails" -> {
                val widgetId = call.argument<String>("widgetId")
                val widgetKind = call.argument<String>("widgetKind")
                widgetId?.let {
                    widgetKind?.let {
                        engagementSDK.fetchWidgetDetails(widgetId, widgetKind,
                            object : LiveLikeCallback<LiveLikeWidget>() {
                                override fun onResponse(widget: LiveLikeWidget?, error: String?) {
                                    uiThreadHandler.post {
                                        widget?.let {
                                            result.success(
                                                toMap(
                                                    JSONObject(
                                                        Gson().toJson(
                                                            widget
                                                        )
                                                    )
                                                )
                                            )
                                        }
                                        error?.let {
                                            result.error("", it, null)
                                        }
                                    }
                                }
                            })
                        return
                    }
                }
                result.error("", "Widget Id and Widget Kind cannot be null or empty", null)
            }
            "fetchUserInteractionDataForWidget" -> {
                val widgetId = call.argument<String>("widgetId")
                val widgetKind = call.argument<String>("widgetKind")
                val programId = call.argument<String>("programId")
                widgetId?.let {
                    widgetKind?.let {
                        programId?.let {
                            val session = sessionMap[programId]
                            engagementSDK.fetchWidgetDetails(widgetId, widgetKind,
                                object : LiveLikeCallback<LiveLikeWidget>() {
                                    override fun onResponse(
                                        widget: LiveLikeWidget?,
                                        error: String?
                                    ) {
                                        uiThreadHandler.post {
                                            error?.let {
                                                result.error("", it, null)
                                            }
                                        }
                                        if (widget?.widgetInteractionUrl != null) {
                                            session?.getWidgetInteraction(widgetId,
                                                widgetKind,
                                                widget.widgetInteractionUrl!!,
                                                object :
                                                    LiveLikeCallback<WidgetUserInteractionBase>() {
                                                    override fun onResponse(
                                                        interaction: WidgetUserInteractionBase?,
                                                        error: String?
                                                    ) {
                                                        uiThreadHandler.post {
                                                            interaction?.let {
                                                                result.success(
                                                                    toMap(
                                                                        JSONObject(
                                                                            Gson().toJson(
                                                                                it
                                                                            )
                                                                        )
                                                                    )
                                                                )
                                                            }
                                                            error?.let {
                                                                result.error("", it, null)
                                                            }
                                                        }
                                                    }
                                                })
                                        } else {
                                            result.error("", "Widget Interaction is null", null)
                                        }
                                    }
                                })
                            return
                        }
                    }
                }
                result.error("", "Program Id,Widget Id and Widget Kind cannot be null or empty", null)
            }

            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}

class LiveLikeFlutterStreamHandler : EventChannel.StreamHandler {
    private var _eventSink: EventChannel.EventSink? = null

    val eventSink: EventChannel.EventSink?
        get() = _eventSink

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        Log.d("LiveLike", "onListen: $events")
        this._eventSink = events
    }

    override fun onCancel(arguments: Any?) {
        this._eventSink = null
    }

}

@Throws(JSONException::class)
fun toMap(`object`: JSONObject): Map<String, Any> {
    val map: MutableMap<String, Any> = HashMap()
    val keysItr = `object`.keys()
    while (keysItr.hasNext()) {
        val key = keysItr.next()
        var value = `object`[key]
        if (value is JSONArray) {
            value = toList(value)
        } else if (value is JSONObject) {
            value = toMap(value)
        }
        map[key] = value
    }
    return map
}

@Throws(JSONException::class)
fun toList(array: JSONArray): List<Any> {
    val list: MutableList<Any> = ArrayList()
    for (i in 0 until array.length()) {
        var value = array[i]
        if (value is JSONArray) {
            value = toList(value)
        } else if (value is JSONObject) {
            value = toMap(value)
        }
        list.add(value)
    }
    return list
}