package com.livelike.flutter.livelike_flutter_sdk

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.DisplayMetrics
import android.view.View
import com.google.gson.Gson
import com.google.gson.JsonParser
import com.livelike.engagementsdk.LiveLikeContentSession
import com.livelike.engagementsdk.core.services.messaging.proxies.LiveLikeWidgetEntity
import com.livelike.engagementsdk.core.services.messaging.proxies.WidgetInterceptor
import com.livelike.engagementsdk.core.services.messaging.proxies.WidgetLifeCycleEventsListener
import com.livelike.engagementsdk.widget.WidgetType
import com.livelike.engagementsdk.widget.view.WidgetView
import com.livelike.engagementsdk.widget.viewModel.WidgetStates
import io.flutter.plugin.common.*
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory
import org.json.JSONObject
import java.util.*
import kotlin.math.roundToInt


class WidgetViewFactory(
    private val plugin: LivelikeFlutterSdkPlugin,
    private val messenger: BinaryMessenger
) : PlatformViewFactory(StandardMessageCodec.INSTANCE) {
    override fun create(context: Context?, viewId: Int, args: Any?): PlatformView {
        val creationParams = args as Map<String?, Any?>?
        println("WidgetViewFactory.create?>> $viewId")
        return FlutterPluginWidgetView(context, plugin, viewId, creationParams, messenger)
    }
}

class FlutterPluginWidgetView(
    context: Context?,
    var plugin: LivelikeFlutterSdkPlugin,
    var viewId: Int,
    creationParams: Map<String?, Any?>?,
    messenger: BinaryMessenger
) : PlatformView, MethodChannel.MethodCallHandler {
    private var pluginWidgetView: PluginWidgetView
    private var widgetView: WidgetView
    private val methodChannel: MethodChannel = MethodChannel(messenger, "widgetView$viewId")
    private val uiThreadHandler: Handler = Handler(Looper.getMainLooper())
    private var currentSession: LiveLikeContentSession? = null
    private val widgetStreamHandler = LiveLikeFlutterStreamHandler()
    private var widgetStreamChannel: EventChannel

    init {
        methodChannel.setMethodCallHandler(this)
        pluginWidgetView = PluginWidgetView(context!!)
        widgetView = pluginWidgetView.findViewById(R.id.sdk_plugin_widget_view)
        widgetStreamChannel = EventChannel(messenger, "widget_stream_$viewId")
        widgetStreamChannel.setStreamHandler(widgetStreamHandler)
        widgetView.widgetLifeCycleEventsListener =
            object : WidgetLifeCycleEventsListener() {
                override fun onUserInteract(widgetData: LiveLikeWidgetEntity) {
                    println("FlutterPluginWidgetView.onUserInteract>${widgetData.kind}")
                }

                override fun onWidgetDismissed(widgetData: LiveLikeWidgetEntity) {
                    println("FlutterPluginWidgetView.onWidgetDismissed>${widgetData.kind}")
                    if (WidgetType.CHEER_METER.event.contains(widgetData.kind) || WidgetType.IMAGE_SLIDER.event.contains(
                            widgetData.kind
                        )
                    ) {
                        uiThreadHandler.post {
                            widgetStreamHandler.eventSink?.success(
                                hashMapOf(
                                    "show" to false,
                                    "data" to toMap(JSONObject(Gson().toJson(widgetData)))
                                )
                            )
                        }
                    }
                }

                override fun onWidgetInteractionCompleted(widgetData: LiveLikeWidgetEntity) {
                    println("FlutterPluginWidgetView.onWidgetInteractionCompleted> ${widgetData.kind}")
                }

                override fun onWidgetPresented(widgetData: LiveLikeWidgetEntity) {
                    widgetView.post {
                        widgetView.getChildAt(0)?.let {
                            it.post {
                                uiThreadHandler.post {
                                    println(
                                        "FlutterPluginWidgetView: Height: ${it.height} => ${widgetData.height} => ${
                                            pxToDp(
                                                widgetView.context,
                                                it.height
                                            )
                                        } => ${widgetData.kind}"
                                    )
                                    widgetStreamHandler.eventSink?.success(
                                        hashMapOf(
                                            "show" to true,
                                            "widget_height" to pxToDp(
                                                widgetView.context,
                                                it.height + when (widgetData.kind == "emoji-slider") {
                                                    true -> 20
                                                    else -> 0
                                                }
                                            ),
                                            "data" to toMap(JSONObject(Gson().toJson(widgetData)))
                                        )
                                    )
                                }
                            }
                        }
                    }
                }

                override fun onWidgetStateChange(
                    state: WidgetStates,
                    widgetData: LiveLikeWidgetEntity
                ) {
                    println("FlutterPluginWidgetView.onWidgetStateChange>> $state >${widgetData.kind}")
                    if (state == WidgetStates.FINISHED) {
                        uiThreadHandler.post {
                            widgetStreamHandler.eventSink?.success(
                                hashMapOf(
                                    "show" to false,
                                    "data" to toMap(JSONObject(Gson().toJson(widgetData)))
                                )
                            )
                        }
                    } else {
                        uiThreadHandler.post {
                            widgetStreamHandler.eventSink?.success(
                                hashMapOf(
                                    "show" to true
                                )
                            )
                        }
                        widgetView.post {
                            widgetView.getChildAt(0)?.let {
                                it.post {
                                    uiThreadHandler.post {
                                        println(
                                            "FlutterPluginWidgetView: Height: ${it.height} => ${widgetData.height} => ${
                                                pxToDp(
                                                    widgetView.context,
                                                    it.height
                                                )
                                            } => ${widgetData.kind}"
                                        )
                                        widgetStreamHandler.eventSink?.success(
                                            hashMapOf(
                                                "show" to true,
                                                "widget_height" to pxToDp(
                                                    widgetView.context,
                                                    it.height + when (widgetData.kind == "emoji-slider") {
                                                        true -> 120
                                                        else -> 0
                                                    }
                                                ),
                                                "data" to toMap(JSONObject(Gson().toJson(widgetData)))
                                            )
                                        )
                                    }
                                }
                            }
                        }
                    }
                }
            }
    }

    override fun getView(): View {
        return pluginWidgetView
    }

    override fun dispose() {
        currentSession?.widgetStream?.unsubscribe(this)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "setSession" -> {
                println("FlutterPluginWidgetView.onMethodCall>> $viewId")
                val programId = call.argument<String>("programId") as String
                plugin.sessionMap[programId]?.let { session ->
                    widgetView.setSession(session)
                    this.currentSession = session
                    session.widgetInterceptor = object : WidgetInterceptor() {
                        override fun widgetWantsToShow(widgetData: LiveLikeWidgetEntity) {
                            println("FlutterPluginWidgetView.widgetWantsToShow>>TRUE >${widgetData.kind}")
                            uiThreadHandler.post {
                                widgetStreamHandler.eventSink?.success(
                                    hashMapOf(
                                        "show" to true,
                                        "data" to toMap(JSONObject(Gson().toJson(widgetData)))
                                    )
                                )
                            }
                            uiThreadHandler.postDelayed({
                                println("FlutterPluginWidgetView.widgetWantsToShow SHOW> ${widgetData.kind}")
                                showWidget()
                            }, 500L)
                        }
                    }
                }
                result.success(null)
            }
            "displayWidget" -> {
                println("FlutterPluginWidgetView.onMethodCall >> ${call.arguments}")
                widgetView.displayWidget(
                    plugin.engagementSDK,
                    JsonParser.parseString(call.arguments.toString()).asJsonObject,
                    showWithInteractionData = true
                )
//                result.success(null)
            }
            "setState" -> {
                val state = call.argument<String>("state") as String
                widgetView.setState(WidgetStates.valueOf(state.toUpperCase(Locale.getDefault())))
                result.success(null)
            }
            "enableDefaultWidgetTransition" -> {
                val enable = call.argument<Boolean>("enable") as Boolean
                widgetView.enableDefaultWidgetTransition = enable
                result.success(null)
            }
            "showDismissButton" -> {
                val show = call.argument<Boolean>("show") as Boolean
                widgetView.showDismissButton = show
                widgetView.showTimer = show
                result.success(null)
            }
        }
    }

    fun pxToDp(context: Context, px: Int): Int {
        val displayMetrics: DisplayMetrics = context.resources.displayMetrics
        return (px / (displayMetrics.xdpi / DisplayMetrics.DENSITY_DEFAULT)).roundToInt()
    }
}