import 'dart:convert';

import 'package:http/http.dart' as http;
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk/src/model/livelike_widget.dart';
import 'package:livelike_flutter_sdk/src/repository/widget_repository.dart';
import 'package:livelike_flutter_sdk/src/repository/program_repository.dart';
import 'package:livelike_flutter_sdk/src/session.dart';
import 'package:livelike_flutter_sdk/src/sponsor.dart';

import 'constants.dart';
import 'model/leaderboard.dart';
import 'repository/user_repository.dart';

class EngagementSDK {
  String? _programDetailTemplate;

  EngagementSDK(this.clientId) {
    _initSDK(clientId, false, null, null);
  }

  EngagementSDK.accessToken(this.clientId, String? accessToken,
      void Function(String accessToken)? onReceiveAccessToken) {
    _initSDK(clientId, true, accessToken, onReceiveAccessToken);
  }

  final String clientId;

  Stream get errorStream => errorEventChannel.receiveBroadcastStream();

  Sponsor get sponsor => Sponsor();

  final _userRepository = UserRepository();
  final _widgetRepository = WidgetRepository();

  String? _accessToken;
  Stream _accessTokenStream = accessTokenEventChannel.receiveBroadcastStream();

  final _programRepository = ProgramRepository();

  Future<void> _initSDK(
      String clientId,
      bool usingAccessToken,
      String? accessToken,
      void Function(String accessToken)? onReceiveAccessToken) async {
    await livelike_method_channel.invokeMethod(initSDKKey, {
      clientIDKey: clientId,
      accessTokenKey: accessToken,
      usingAccessTokenKey: usingAccessToken
    });
    if (usingAccessToken) {
      _accessToken = accessToken;
      _accessTokenStream.listen((event) {
        _accessToken = event;
        onReceiveAccessToken?.call(event);
      });
    }
  }

  Future<Session> createSession(String programId) async {
    Map result = await (livelike_method_channel
        .invokeMethod(createSessionKey, {programIDKey: programId}));
    if (_programDetailTemplate == null) {
      await _fetchApplicationData();
    }
    return Session(result[programIDKey], result[chatRoomIdKey], _accessToken,
        _programDetailTemplate!);
  }

  Future<ChatSession> createChatSession(String chatRoomId) async {
    await livelike_method_channel
        .invokeMethod(createChatSessionKey, {chatRoomIdKey: chatRoomId});
    return ChatSession(chatRoomId);
  }

  Future<List<LeaderBoard>> getLeaderBoards(String programId) async {
    List result = await livelike_method_channel
        .invokeMethod(getLeaderBoardsKey, {programIDKey: programId});
    return result.map((e) => LeaderBoard.fromJson(e)).toList();
  }

  Future<LeaderBoard> getLeaderBoardDetails(String leaderBoardId) async {
    Map result = await livelike_method_channel.invokeMethod(
        getLeaderBoardDetailsKey, {leaderBoardIdKey: leaderBoardId});
    return LeaderBoard.fromJson(result);
  }

  Future<LeaderBoardEntry> getLeaderBoardEntryForProfile(
      String leaderBoardId, String profileId) async {
    Map result = await livelike_method_channel.invokeMethod(
        getLeaderBoardEntryForProfileKey,
        {leaderBoardIdKey: leaderBoardId, profileIdKey: profileId});
    return LeaderBoardEntry.fromJson(result);
  }

  Future<LeaderBoardEntry> getLeaderBoardEntryForCurrentUserProfile(
      String leaderBoardId) async {
    Map result = await livelike_method_channel.invokeMethod(
        getLeaderBoardEntryForCurrentUserProfileKey,
        {leaderBoardIdKey: leaderBoardId});
    return LeaderBoardEntry.fromJson(result);
  }

  Future<LeaderBoardEntryResult> getEntriesForLeaderBoard(
      String leaderBoardId, LiveLikePagination likePagination) async {
    Map result = await livelike_method_channel
        .invokeMethod(getEntriesForLeaderBoardKey, {
      leaderBoardIdKey: leaderBoardId,
      paginationKey: likePagination.toString().split(".")[1]
    });
    return LeaderBoardEntryResult.fromJson(result);
  }

  Future<LiveLikeWidget> fetchWidgetDetails(
      String widgetId, String widgetKind) async {
    return _widgetRepository.getWidgetDetails(widgetId, widgetKind);
  }

  Future<WidgetUserInteraction?> fetchWidgetInteraction(
      String widgetId, String widgetKind) async {
    print("Token: $_accessToken");

    final profile = await _userRepository.getUserProfile(clientId, _accessToken!);
    final widget = await _widgetRepository.getWidgetInteractionDetails(
        widgetId,
        widgetKind,
        profile.id!,
        _accessToken!);
    return widget;
  }

  Future<void> _fetchApplicationData() async {
    final response =
        await http.get(Uri.parse("${baseUrl}applications/$clientId"));
    final Map<String, dynamic> data = jsonDecode(response.body);
    _programDetailTemplate = data["program_detail_url_template"];
  }

  ProgramListResponse? _programListResponse;

  Future<List<Program>> fetchListOfProgram(
      LiveLikePagination pagination) async {
    switch (pagination) {
      case LiveLikePagination.first:
        _programListResponse =
            await _programRepository.getListofPrograms(clientId);
        break;
      case LiveLikePagination.next:
        _programListResponse = await _programRepository
            .getListofPrograms(clientId, url: _programListResponse?.next);
        break;
      case LiveLikePagination.previous:
        _programListResponse = await _programRepository
            .getListofPrograms(clientId, url: _programListResponse?.previous);
        break;
    }
    return _programListResponse?.results ?? <Program>[];
  }

  Future<Program> fetchProgram(String programId) async {
    return _programRepository.getProgramDetails(programId);
  }
}
