import 'dart:convert';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:livelike_flutter_sdk/src/constants.dart';
import 'package:livelike_flutter_sdk/src/model/livelike_widget.dart';
import 'package:livelike_flutter_sdk/src/session.dart';

class WidgetView extends StatefulWidget {
  WidgetView(
      {Key? key,
      this.session,
      this.widgetHeight = 350,
      this.widgetListener,
      this.showWidgetViewWithDefaultHeight = false,
      required this.onWidgetViewInit})
      : super(key: key);
  final Session? session;
  final double widgetHeight;
  final void Function(String widgetId, String widgetKind)? widgetListener;
  final void Function() onWidgetViewInit;
  final bool showWidgetViewWithDefaultHeight;

  @override
  WidgetViewState createState() => WidgetViewState();
}

class WidgetViewState extends State<WidgetView>
    with AutomaticKeepAliveClientMixin {
  final Map<String, dynamic> creationParams = <String, dynamic>{};
  bool _showWidget = true;
  double? widgetHeight;

  MethodChannel? _channel;
  EventChannel? widgetStreamChannel;

  Stream<LiveLikeWidget>? get widgetStream => widgetStreamChannel
      ?.receiveBroadcastStream()
      .map((event) => LiveLikeWidget.fromJson(event["data"] ?? {})
        ..show = event["show"]
        ..widget_height = event["widget_height"]?.toDouble());

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: widget.showWidgetViewWithDefaultHeight
          ? (widgetHeight ?? widget.widgetHeight)
          : _showWidget
              ? (widgetHeight ?? widget.widgetHeight)
              : 0,
      child: Platform.isAndroid
          ? PlatformViewLink(
              surfaceFactory: (context, controller) {
                return AndroidViewSurface(
                  controller: controller as AndroidViewController,
                  gestureRecognizers: const <
                      Factory<OneSequenceGestureRecognizer>>{},
                  hitTestBehavior: PlatformViewHitTestBehavior.opaque,
                );
              },
              onCreatePlatformView: (params) {
                print("widget_view>>>> ${params.id}");
                widgetStreamChannel =
                    EventChannel('widget_stream_${params.id}');
                return PlatformViewsService.initSurfaceAndroidView(
                  id: params.id,
                  viewType: widgetViewTypeKey,
                  layoutDirection: TextDirection.ltr,
                  creationParams: creationParams,
                  creationParamsCodec: const StandardMessageCodec(),
                )
                  ..addOnPlatformViewCreatedListener((id) async {
                    print("widget_view: $id");
                    _channel = new MethodChannel('$widgetViewTypeKey$id');
                    if (widget.session != null) {
                      await _channel?.invokeMethod(setSessionKey,
                          {programIDKey: widget.session!.programId});
                    }
                    params.onPlatformViewCreated(id);
                    _streamListener();
                  })
                  ..create();
              },
              viewType: widgetViewTypeKey)
          : Platform.isIOS
              ? UiKitView(
                  viewType: widgetViewTypeKey,
                  layoutDirection: TextDirection.ltr,
                  creationParams: creationParams,
                  creationParamsCodec: const StandardMessageCodec(),
                  gestureRecognizers: const <
                      Factory<OneSequenceGestureRecognizer>>{},
                  hitTestBehavior: PlatformViewHitTestBehavior.opaque,
                  onPlatformViewCreated: (id) async {
                    widgetStreamChannel = EventChannel('widget_stream_$id');
                    _channel = new MethodChannel('$widgetViewTypeKey$id');
                    if (widget.session != null) {
                      await _channel?.invokeMethod(setSessionKey,
                          {programIDKey: widget.session!.programId});
                    }
                    _streamListener();
                  },
                )
              : SizedBox.shrink(),
    );
  }

  Future<void> displayWidget(LiveLikeWidget liveLikeWidget) async {
    setState(() {
      _showWidget = true;
    });
    await _channel?.invokeMethod(
        "displayWidget", jsonEncode(liveLikeWidget.toJson()));
  }

  Future<void> setWidgetState(WidgetState state) async {
    await _channel
        ?.invokeMethod("setState", {"state": state.toString().split(".")[1]});
  }

  Future<void> enableDefaultWidgetTransition(bool enable) async {
    if (Platform.isAndroid)
      await _channel
          ?.invokeMethod("enableDefaultWidgetTransition", {"enable": enable});
  }

  Future<void> showDismissButton(bool show) async {
    if (Platform.isAndroid)
      await _channel?.invokeMethod("showDismissButton", {"show": show});
  }

  void _streamListener() async {
    await Future.delayed(const Duration(seconds: 2));
    if (mounted)
      setState(() {
        _showWidget = false;
      });
    widgetStream?.listen((event) async {
      if (mounted) {
        if (event.id != null && event.kind != null) {
          widget.widgetListener?.call(event.id!, event.kind!);
        }
        print(
            "CHECK Show: ${event.show} , Height:${event.widget_height}> ${MediaQuery.of(context).devicePixelRatio}");
        setState(() {
          _showWidget = (event.show ?? false);
          if (event.widget_height != null) {
            widgetHeight =
                (event.widget_height ?? 0) + (Platform.isIOS ? 30 : 5);
            if ((widgetHeight ?? 0) < 100) {
              widgetHeight = 130;
            }
          } else
            widgetHeight = null;
        });
      }
    });
    widget.onWidgetViewInit.call();
  }

  @override
  void dispose() {
    super.dispose();
    print("Disponse :");
  }

  @override
  bool get wantKeepAlive => true;
}
